﻿/*!
 *      @date   2011/09/08
 *      @file   dhwr_setting.h
 *      @author jskim
 *
 *      Copyright (c) 2016 by SELVAS AI Inc. All rights reserved.
 */

#ifndef __DHWR_SETTING_OBJECT_H__
#define __DHWR_SETTING_OBJECT_H__ 1

#include "dhwr_define.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @defgroup SettingObject SettingObject APIs
 * @ingroup  Main
 * @brief    엔진 동작을 결정하는 SettingObject관련 함수
 */

/*@{*/

/*!
 *  @brief 언어 리스트
 */
typedef enum
{
  // Latin
  //! 영어(미국)
  DLANG_ENGLISH,
  //! 영어(영국)
  DLANG_ENGLISH_UK,
  //! 영어(캐나다)
  DLANG_ENGLISH_CA,
  //! 알바니아어
  DLANG_ALBANIAN,
  //! 오스트리아어
  DLANG_AUSTRIA,
  //! 바스크어(스페인)
  DLANG_BASQUE,
  //! 카탈로니아어
  DLANG_CATALAN,
  //! 크로아티아어
  DLANG_CROATIAN,
  //! 체코어
  DLANG_CZECH,
  //! 덴마크어
  DLANG_DANISH,
  //! 네덜란드어(네덜란드)
  DLANG_DUTCH,
  //! 에스토니아어
  DLANG_ESTONIAN,
  //! 핀란드어
  DLANG_FINNISH,
  //! 프랑스어
  DLANG_FRENCH,
  //! 스페인어(스페인)
  DLANG_SPANISH,
  //! 스페인어(맥시코)
  DLANG_SPANISH_MX,

  //! 헝가리어
  DLANG_HUNGARIAN,
  //! 아이슬란드어
  DLANG_ICELANDIC,
  //! 이탈리아어
  DLANG_ITALIAN,
  //! 라트비아어
  DLANG_LATVIAN,
  //! 리투아니아어
  DLANG_LITHUANIAN,
  //! 노르웨이어(보크말)
  DLANG_BOKMAL,
  //! 폴란드어
  DLANG_POLISH,
  //! 포르투갈어(포르투갈)
  DLANG_PORTUGUESE,
  //! 포르투갈어(브라질)
  DLANG_PORTUGUESEB,
  //! 루마니아어
  DLANG_ROMANIAN,
  //! 슬로바키아어
  DLANG_SLOVAK,
  //! 슬로베니아어
  DLANG_SLOVENIAN,
  //! 스웨덴어
  DLANG_SWEDISH,
  //! 터키어
  DLANG_TURKISH,
  //! 베트남어
  DLANG_VIETNAMESE,
  //! 벨로루시어
  DLANG_BELARUSIAN,
  //! 불가리아어
  DLANG_BULGARIAN,
  // Greek
  //! 그리스어
  DLANG_GREEK,
  // Cyrillic
  //! 카자흐어
  DLANG_KAZAKH,
  //! 마케도니아어
  DLANG_MACEDONIAN,
  //! 몽골리아어
  DLANG_MONGOLIAN,
  //! 러시아어
  DLANG_RUSSIAN,
  //! 우크라이나어
  DLANG_UKRAINIAN,
  // Latin, Cyrillic
  //! 세르비아어
  DLANG_SERBIAN,
  //! 타타르어(러시아)
  DLANG_TATAR,
  // Oceania
  //! 피지어(피지 공화국)
  DLANG_FIJIAN,
  //! 마오리어(뉴질랜드)
  DLANG_MAORI,
  //! 사모아어
  DLANG_SAMOAN,
  //! 타히티어(프랑스령 타히티섬)
  DLANG_TAHITIAN,
  //! 통가어(통가)
  DLANG_TONGAN,
  // Africa
  //! 오로모어(케냐)
  DLANG_OROMO,
  //! 소토어(남아프리카 공화국)
  DLANG_SOTHO,
  //! 스와힐리어(케냐)
  DLANG_SWAHILI,
  //! 스와트어(스와질란드)
  DLANG_SWATI,
  //! 호사어(남아프리카 공화국)
  DLANG_XHOSA,
  //! 줄루어(남아프리카)
  DLANG_ZULU,

  //! 인도네시아어
  DLANG_INDONESIAN,
  //! 말레이어
  DLANG_MALAY,
  //! 갈리시아어(스페인)
  DLANG_GALICIAN,
  //! 독일어(독일)
  DLANG_GERMANY,
  //! 아일랜드어
  DLANG_IRISH,
  //! 남아공 공용어(남아프리카)
  DLANG_AFRIKAANS,
  //! 노르웨이어(노르웨이)
  DLANG_NYNORSK,
  //! 네덜란드어(벨기에)
  DLANG_DUTCH_BE,
  //! 아제르바이잔어
  DLANG_AZERBAIJANI,

  // CJK
  //! 한국어
  DLANG_KOREAN    = 101,
  //! 중국어(중국)
  DLANG_CHINA,
  //! 중국어(대만)
  DLANG_TAIWAN,
  //! 중국어(홍콩, 중국 특별행정구)
  DLANG_HONGKONG,
  //! 일본어
  DLANG_JAPANESE,
  // ARABIC
  //! 아랍어
  DLANG_ARABIC,
  //! 페르시아어(이란)
  DLANG_FARSI,
  //! 우르두어(파키스탄)
  DLANG_URDU,
  // Devanagari
  //! 힌디어
  DLANG_HINDI,
  //! 벵골어(방글라데시)
  DLANG_BENGALI,
  // etc
  //! 히브리어(이스라엘)
  DLANG_HEBREW,
  //! 태국어
  DLANG_THAI,

  // max
  DLANG_MAX
} DHWR_LANG;

/*!
 * @brief  인식할 언어의 상세 설정을 위해서 사용할 값이 정의되어 있음
 * @details  예를 들어, @ref DHWR_LANG "Language"가 영어(#DLANG_ENGLISH)일 경우, 대문자만 인식하거나, 소문자만 인식하거나, 또는 숫자만 인식하게 하는 등의 설정이 가능함
 * @code
 * // 영어 대문자만 인식
 * DHWRAddLanguage(settingObject, DLANG_ENGLISH, DTYPE_UPPERCASE);
 * // 영어 소문자만 인식
 * DHWRAddLanguage(settingObject, DLANG_ENGLISH, DTYPE_LOWERCASE);
 * // 영어 대문자, 소문자 동시 인식
 * DHWRAddLanguage(settingObject, DLANG_ENGLISH, DTYPE_UPPERCASE | DTYPE_LOWERCASE);
 * // 숫자만 인식
 * DHWRAddLanguage(settingObject, DLANG_ENGLISH, DTYPE_NUMERIC);
 * // 기호만 인식
 * DHWRAddLanguage(settingObject, DLANG_ENGLISH, DTYPE_SIGN);
 *
 * // 한국어 자음만 인식
 * DHWRAddLanguage(settingObject, DLANG_KOREAN, DTYPE_CONSONANT);
 * // 한국어 모음만 인식
 * DHWRAddLanguage(settingObject, DLANG_KOREAN, DTYPE_VOWEL);
 * // 한국어 글자, 숫자 동시 인식
 * DHWRAddLanguage(settingObject, DLANG_KOREAN, DTYPE_KOREAN | DTYPE_NUMERIC);
 * @endcode
 * - @ref DHWR_TYPE "Type"을 기호(#DTYPE_SIGN)로 설정할 경우, 인식 가능한 기호 목록은 다음과 같음
 |Hex Code|Character|
 |------------|-----------|
 |0021 ~ 002F|! " # $ % & ' ( ) * + , - . /|
 |003A ~ 0040|: ; < = > ? @|
 |005B ~ 0060|[ \\ ] ^ _ `|
 |007B ~ 007E|{ \| } ~|
 */
typedef enum
{
  //! 타입 없음
  DTYPE_NONE        = BIT_FLAG(0),
  //! 대문자
  DTYPE_UPPERCASE   = BIT_FLAG(1),
  //! 소문자
  DTYPE_LOWERCASE   = BIT_FLAG(2),
  //! 자음
  DTYPE_CONSONANT   = BIT_FLAG(3),
  //! 모음
  DTYPE_VOWEL       = BIT_FLAG(4),
  //! 성조
  DTYPE_TONE        = BIT_FLAG(5),
  //! 숫자
  DTYPE_NUMERIC     = BIT_FLAG(6),
  //! 기호
  DTYPE_SIGN        = BIT_FLAG(7),
  // JAPANESE
  //! 히라가나
  DTYPE_HIRAGANA    = BIT_FLAG(8),
  //! 카타카나
  DTYPE_KATAKANA    = BIT_FLAG(9),
  //! 칸지
  DTYPE_KANJI       = BIT_FLAG(10),
  // CHINA, TAIWAN, HONGKONG
  //! 간체
  DTYPE_SIMP        = BIT_FLAG(11),
  //! 번체
  DTYPE_TRAD        = BIT_FLAG(12),
  //! 부수
  DTYPE_RADICAL     = BIT_FLAG(13),
  // SERBIAN, TATAR
  //! 라틴 문자
  DTYPE_LATIN       = BIT_FLAG(14),
  //! 키릴 문자
  DTYPE_CYRILLIC    = BIT_FLAG(15),
  // ARABIC, FARSI, URDU
  //! 아랍 문자
  DTYPE_ARABIC      = BIT_FLAG(16),
  // HEBREW
  //! 히브리 문자
  DTYPE_HEBREW      = BIT_FLAG(17),
  // BENGALI, HINDI
  //! 데바나가리 문자
  DTYPE_DEVANAGARI  = BIT_FLAG(18),
  // KOREAN
  //! 한글
  DTYPE_KOREAN      = BIT_FLAG(19),
  //! 한자
  DTYPE_HANJA       = BIT_FLAG(20),
  // gesture for editing (space, backspace 등)
  //! 제스처
  DTYPE_GESTURE     = BIT_FLAG(21),
} DHWR_TYPE;

/*!
*  @brief 인식 타입 종류
*/
typedef enum
{
  //! 낱자 인식
  SINGLECHAR,
  //! 여러 글자 인식
  MULTICHAR,
  //! 겹쳐 쓴 글자 인식
  OVERLAPCHAR,
  //! 여러 줄로 된 글자 인식
  MULTILINE,
} DHWR_RECOGMODE;

//! SettingObject Handle
typedef const void* DHWRSettingObject;

/*!
 *  @brief 설정 오브젝트를 생성한다
 *  @return @ref DHWRSettingObject
 */
DHWR_API DHWRSettingObject DHWRCreateSettingObject(void);

/*!
 *  @brief 설정 오브젝트를 제거한다
 *  @param [in] setting 설정 오브젝트
 *  @return
 */
DHWR_API void DHWRDestroySettingObject(DHWRSettingObject setting);

/*!
 *  @brief 필기 인식 모드를 설정한다(낱자, 연속)
 *  @param [in] setting 설정 오브젝트
 *  @param [in] mode @ref DHWR_RECOGMODE "인식 모드"
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRSetRecognitionMode(DHWRSettingObject setting,
  const DHWR_RECOGMODE mode);

/*!
 *  @brief 인식 후보 출력 크기를 설정한다(낱자, 연속)
 *  @param [in] setting 설정 오브젝트
 *  @param [in] size_cand 후보수
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRSetCandidateSize(DHWRSettingObject setting,
  const int size_cand);

/*!
 *  @brief 인식할 언어를 추가 한다
 *  @param [in] setting 설정 오브젝트
 *  @param [in] lang 언어 값
 *  @param [in] option @ref DHWR_TYPE "언어에 따른 옵션"
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRAddLanguage(DHWRSettingObject setting,
  const DHWR_LANG lang, const DHWR_TYPE option);

/*!
 *  @brief 설정된 언어 개수를 가져온다
 *  @param [in] setting 설정 오브젝트
 *  @param [out] size 언어 개수
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRGetLanguageSize(DHWRSettingObject setting, int* size);

/*!
 *  @brief 설정된 언어를 초기화 한다
 *  @param [in] setting 설정 오브젝트
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRClearLanguage(DHWRSettingObject setting);

/*!
 *  @brief 인식할 심볼을 정의
 *  @param [in] setting 설정 오브젝트
 *  @param [in] charset 인식할 심볼의 리스트
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRSetUserCharSet(DHWRSettingObject setting,
  const unsigned short* charset);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif // __DHWR_SETTING_OBJECT_H__
